#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" OpenAI: Generate Inflection Prompts """


from typing import List
from typing import Optional

from baseblock import BaseObject

from openai_helper import chat

from owl_builder.core import OpenAICache
from owl_builder.synonyms.dmo import SynonymPromptGenerator


class GenerateEnglishSynonyms(BaseObject):
    """ OpenAI: Generate Inflection Prompts """

    def __init__(self):
        """
        Created:
            29-Mar-2023
            craigtrim@gmail.com
            *   https://github.com/craigtrim/owl-builder/issues/10
        """
        BaseObject.__init__(self, __name__)
        cache = OpenAICache()
        self._exists_in_cache = cache.exists
        self._write_to_cache = cache.write
        self._read_from_cache = cache.read

        self._prompt_generator = SynonymPromptGenerator().process

    def process(self,
                entity_name: str,
                entity_context: Optional[str] = None,
                desired_synonym_size: Optional[str] = None) -> Optional[str]:

        prompt = self._prompt_generator(
            entity_name=entity_name,
            entity_context=entity_context,
            desired_synonym_size=desired_synonym_size)

        result = chat(
            input_prompt=prompt,
            messages=None,
            remove_emojis=True,
            model="gpt-4")

        if not result or not len(result):
            return None

        def split_results() -> List[str]:
            if '\n' in result:
                return result.split('\n')
            if ',' in result:
                return result.split(',')
            return result

        results = [
            x.strip() for x in split_results()
        ]

        results = [
            x for x in results if x and len(x)
        ]

        return sorted(results, reverse=False)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Generate an Ontology Graph Node in TTL Format """


from typing import List
from typing import Optional
from functools import partial

from datetime import datetime
from baseblock import BaseObject
from baseblock import TextUtils

from owl_builder.synonyms.svc import GenerateEnglishSynonyms
from owl_builder.synonyms.svc import GenerateEnglishInflections

from rdflib.graph import Graph
from rdflib import Graph, URIRef, Literal
from rdflib.namespace import OWL, RDF, RDFS
from rdflib import Graph, URIRef, Literal
from rdflib.namespace import RDFS, XSD


class GenerateTTLNode(BaseObject):
    """ Generate an Ontology Graph Node in TTL Format """

    def __init__(self,
                 model_name: str = "gpt-4"):
        """ Change Log

        Created:
            28-Mar-2023
            craigtrim@gmail.com
            *   https://github.com/craigtrim/owl-builder/issues/8

        model_name (str, optional). defaults to "GPT-4"
        """
        BaseObject.__init__(self, __name__)
        self._model_name = model_name

        self._generate_synonyms = partial(
            GenerateEnglishSynonyms().process,
            desired_synonym_size="trigram")

        self._generate_inflections = partial(
            GenerateEnglishInflections().process,
            model=self._model_name)

    @staticmethod
    def _to_identifier(entity_name: str) -> str:

        entity_name = entity_name.upper().strip()
        entity_name = entity_name.replace(' ', '_')

        for item in ["'"]:
            entity_name = entity_name.replace(item, "").strip()

        return entity_name

    def _version(self) -> str:
        text = "Generated on #TIME using #MODEL"

        text = text.replace('#TIME', str(datetime.now())[:-10])
        text = text.replace('#MODEL', self._model_name.upper().strip())

        return text

    def _synonyms(self,
                  entity_name: str,
                  entity_context: Optional[str] = None) -> Optional[List[str]]:

        master = set()

        def update(results: Optional[List[str]]):
            if results:
                [master.add(x) for x in results]

        update(self._generate_synonyms(
            entity_name=entity_name,
            entity_context=entity_context))

        update(self._generate_inflections(
            input_text=entity_name))

        if not len(master):
            return None

        return sorted(master, key=len, reverse=True)

    def process(self,
                entity_name: str,
                entity_context: Optional[str] = None,
                namespace: Optional[str] = "http://graffl.ai/test") -> str:

        entity_id = self._to_identifier(entity_name)
        entity_label = TextUtils.title_case(entity_name)

        g = Graph()

        g.add((
            URIRef(f"{namespace}/{entity_id}"),
            RDF.type,
            OWL.Class
        ))

        g.add((
            URIRef(f"{namespace}/{entity_id}"),
            RDFS.label,
            Literal(entity_label, datatype=XSD.string)
        ))

        g.add((
            URIRef(f"{namespace}/{entity_id}"),
            OWL.versionInfo,
            Literal(self._version(), datatype=XSD.string)
        ))

        synonyms = self._synonyms(
            entity_name=entity_name,
            entity_context=entity_context)

        if synonyms and len(synonyms):
            for synonym in synonyms:
                g.add((
                    URIRef(f"{namespace}/{entity_id}"),
                    RDFS.seeAlso,
                    Literal(synonym, datatype=XSD.string)
                ))

        result = g.serialize(format="turtle")

        normalized = [
            f"### {namespace}/{entity_id}"
        ]

        lines = result.split('\n')

        lines = [
            line.strip() for line in lines
            if line and len(line)
        ]

        lines = [
            line for line in lines
            if not line.startswith('@prefix')
        ]

        for line in lines:

            if line.startswith('<http:'):
                line = f":{entity_label} rdf:type owl:Class ;"

            if line.startswith("rdfs:") or line.startswith("owl:"):
                line = f"  {line}"
            elif line.startswith('"'):
                line = f"     {line}"

            normalized.append(line)

        return '\n'.join(normalized)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Use OpenAI to Generate Intent Names """


from typing import Optional
from openai_helper import chat

from baseblock import (
    BaseObject,
    Stopwatch,
    Enforcer,
)


class GenerateIntentName(BaseObject):
    """ Use OpenAI to Generate Intent Names """

    def __init__(self):
        """ Change Log

        Created:
            27-Mar-2023
            craigtrim@gmail.com
            *   https://github.com/craigtrim/owl-builder/issues/6
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    def _prompt(input_text: str) -> str:
        return """
Create intent definitions for each string.  Use the same format as the examples.  The intent definition should be 3 words delimited by underscores.

Examples:
WHAT_YOUR_DAY: What does a typical day in the life of you look like?
WHAT_YOUR_COLOR: I am curious to know which color holds a special place in your heart?
WHAT_YOUR_BOOK: What is your favorite book?
ARE_YOU_FEMALE: Are you a woman?
ARE_YOU_HUMAN: Are you human?
ARE_YOU_MALE: Are you a man?
DO_YOU_DATE: Will you go out on a date with me?
DO_YOU_FUN: What do you do for fun?
DO_YOU_NETFLIX: Do you watch Netflix?
GIVE_ME_HINT: Can you give me a hint?
PROPHESY_ME_FUTURE: What will I be when I grow up?

Input: #INPUT_TEXT""".replace('#INPUT_TEXT', input_text).strip()

    def process(self,
                input_text: str) -> Optional[str]:

        sw = Stopwatch()

        if self.isEnabledForDebug:
            Enforcer.is_str(input_text)

        intent_name = chat(
            input_prompt=self._prompt(input_text),
            messages=None,
            remove_emojis=True)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Intent Naming Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tIntent Name: {intent_name}"]))

        return intent_name



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Generate Additional QA Pairs from a Given Example """


from functools import lru_cache
from typing import List

from baseblock import Enforcer
from baseblock import Stopwatch
from baseblock import BaseObject

from openai_helper import chat


class GenerateQAPairs(BaseObject):
    """ Generate Additional QA Pairs from a Given Example """

    def __init__(self):
        """
        Created:
            28-Mar-2023
            craigtrim@gmail.com
            *   https://github.com/craigtrim/owl-builder/issues/5
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    @lru_cache
    def _prompt(question: str,
                answer: str) -> str:
        prompt = """I will give you a question and answer pair.  Please generate 10 more.  You may rephrase both the Question and the Answer, but don't change the meaning of either the Question or the Answer.
----------------------------------
Question: #QUESTION
Answer: #ANSWER
----------------------------------""".strip()

        prompt = prompt.replace('#QUESTION', question)
        prompt = prompt.replace('#ANSWER', answer)

        return prompt

    def process(self,
                question: str,
                answer: str) -> List[str]:

        sw = Stopwatch()

        if self.isEnabledForDebug:
            Enforcer.is_str(question)
            Enforcer.is_str(answer)

        results = chat(
            input_prompt=self._prompt(
                question=question,
                answer=answer),
            messages=None,
            remove_emojis=True)

        qa_pairs = [
            x for x in results.split('Question:')
            if x and len(x)
        ]

        qa_pairs = [
            [
                y.strip() for y in x.split('Answer:')
                if y and len(y)
            ]
            for x in qa_pairs
        ]

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "QA Pair Generation Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tQA Pairs: {qa_pairs}"]))

        return qa_pairs



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Keyterms using an OpenAI Model """

import hashlib
from typing import List

from baseblock import Enforcer
from baseblock import Stopwatch
from baseblock import BaseObject

from openai_helper import chat
from owl_builder.core import OpenAICache


class OpenAIKeytermExtractor(BaseObject):
    """ Extract Keyterms using an OpenAI Model """

    def __init__(self):
        """
        Created:
            28-Mar-2023
            craigtrim@gmail.com
            *   https://github.com/craigtrim/owl-builder/issues/5
        """
        BaseObject.__init__(self, __name__)
        cache = OpenAICache()
        self._exists_in_cache = cache.exists
        self._write_to_cache = cache.write
        self._read_from_cache = cache.read

    @staticmethod
    def _prompt(input_text: str) -> str:
        return f"Some text is provided below. Given the text, extract up to 10 keywords from the text. Avoid stopwords.---------------------\\n{input_text}\\n---------------------\\nProvide keywords in the following comma-separated format: \'KEYWORDS: <keywords>\'\\n"

    @staticmethod
    def _cleanse(input_text: str) -> str:
        if input_text.endswith('.'):
            input_text = input_text[:-1]

        return input_text

    def process(self,
                input_text: str) -> List[str]:

        sw = Stopwatch()

        if self.isEnabledForDebug:
            Enforcer.is_str(input_text)

        if self._exists_in_cache(input_text):
            return self._read_from_cache(input_text)

        keyterms = chat(
            input_prompt=self._prompt(input_text),
            messages=None,
            remove_emojis=True)

        if keyterms:

            if keyterms.startswith('KEYWORDS:'):
                keyterms = keyterms[9:].strip().split(',')

            keyterms = [
                x.strip() for x in keyterms
            ]

            keyterms = [
                self._cleanse(x) for x in keyterms
                if x and len(x)
            ]

        self._write_to_cache(data=keyterms, file_name=input_text)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Keyword Extraction Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tExtracted Terms: {keyterms}"]))

        return keyterms



from typing import List
from typing import Optional

from .bp import *
from .dmo import *
from .svc import *
from .svc.generate_english_synonyms import GenerateEnglishSynonyms
from .svc.generate_english_inflections import GenerateEnglishInflections

synonym_generator = GenerateEnglishSynonyms().process
inflection_generator = GenerateEnglishInflections().process


def generate_synonyms(entity_name: str,
                      entity_context: Optional[str] = None,
                      desired_synonym_size: Optional[str] = None) -> Optional[List[str]]:
    """ Generate Synonyms for an Entity

    Args:
        entity_name (str): the name of any entity
        entity_context (Optional[str], optional): the context in which this entity appears. Defaults to None.
            this will improve the precision of the synonyms
        desired_synonym_size (Optional[str], optional): the size of the output synonyms: Unigram, Bigram, Trigram. Defaults to None.

    Returns:
        Optional[List[str]]: the generated synonyms
    """
    return synonym_generator(
        entity_name=entity_name,
        entity_context=entity_context,
        desired_synonym_size=desired_synonym_size)


def generate_inflections(entity_name: str,
                         model: Optional[str] = "gpt-4") -> Optional[List[str]]:
    """ Generate Inflections for an Entity

    Args:
        entity_name (str): the name of any entity
        model (Optional[str], optional): the model to use for genration. Defaults to "gpt-4".

    Returns:
        Optional[List[str]]: the generated inflections
    """
    return inflection_generator(
        model=model,
        input_text=entity_name)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Use OpenAI to Generate Intent Names """


import os
os.environ['USE_OPENAI'] = str(True)
os.environ['OPENAI_KEY'] = 'gAAAAABiH-eZKbScaS9reXABcCVeRA-VK7rbh-ZBzH72tfzRjTHIH6y5DmcFPxs1Hbf5suJufyD6Z_WhL4h1N1s_BBGpV5JqGZpVCPoB-dAPIFz6gE3uEgUMv_le5RYej5jnZawccsOSKA1RWWpC-CVTcn80S4LehA=='
os.environ['OPENAI_ORG'] = 'gAAAAABiH-0FMWAwMybowrACJ6GPkC91E4DgaV2lJoextMR7U4O5DjB_pw9jBUusuCdH9KEUXp-3Iq-Fni1X-eE6ulSg8JEKtZs-bClX_D-2DyvYpv65iPs='

from pprint import pprint
from openai_helper import chat
from collections import defaultdict
from owl_builder import extract_keyterms
from owl_builder.keyterms.bp import AutoTaxoOrchestrator

from typing import (
    List,
    Optional
)

from baseblock import (
    BaseObject,
    Stopwatch,
    Enforcer,
)

autotaxo_orchestrator = AutoTaxoOrchestrator()


def _keyterms(questions: List[str]) -> List[str]:

    d_questions = {}
    d_questions_rev = defaultdict(list)

    for question in questions:

        terms = autotaxo_orchestrator.keyterms(
            input_text=question,
            use_openai=True,
            use_terms=False,
            use_keyterms=False,
            use_ngrams=False,
            use_nounchunks=False)

        print(question, terms)

        d_questions[question] = terms

    for question in d_questions:
        for term in d_questions[question]:
            d_questions_rev[term].append(question)

    return d_questions, d_questions_rev


def main():
    d_questions, d_questions_rev = _keyterms([
        "What is a typical day in your life like?",
        "Could you outline what you do from morning to night?",
        "Could you tell me about your standard daily routine?",
        "Can you give me a rundown of your daily activities?",
        "What are the tasks you typically complete in a day?",
        "Could you give me a rundown of your daily routine?",
        "Can you give me an idea of your daily activities?",
        "Could you tell me about your typical day-to-day?",
        "How do you go about your day-to-day activities?",
        "How do you typically spend your time each day?",
        "Could you walk me through your daily schedule?",
        "What do you usually do on a day-to-day basis?",
        "What is an ordinary day like for you?",
        "Can you describe your regular daily routine?",
        "How do you fill your hours on a regular day?",
        "What do you typically do throughout the day?",
        "What’s your schedule like on a regular day?",
        "Walk me through a typical day in your life.",
        "What's the usual routine for you each day?",
        "What is a typical day like for you?",
        "What is a typical day like for you?",
        "Can you paint a picture of your usual day?",
        "What does an ordinary day entail for you?",
        "Can you tell me about your daily routine?",
        "What is an usual day like for you?",
        "What does a typical day involve for you?",
        "Give me an idea of what you do in a day.",
        "How do you pass the time during the day?",
        "Can you describe your typical day to me?",
        "How do you structure your time each day?",
        "What’s a typical day like for you?",
        "Describe a day in your typical routine.",
        "What do you do from morning till night?",
        "Tell me about your day-to-day routine.",
        "Walk me through a typical day for you.",
        "What do you usually do during the day?",
        "How do you spend your day-to-day life?",
        "What does your average day consist of?",
        "How do you usually structure your day?",
        "Can you describe a normal day for you?",
        "What does a day in your life involve?",
        "What's the general plan for your day?",
        "Can you detail a typical day for me?",
        "How do you typically spend your day?",
        "Can you describe your daily routine?",
        "How do you usually spend your time?",
        "Tell me about your day-to-day life.",
        "Describe your usual daily schedule.",
        "What is a regular day like for you?",
        "Can you describe your typical day?",
        "How do you spend your average day?",
        "Can you summarize your day for me?",
        "How do you spend your day usually?",
        "What’s a typical day in your life?",
        "Describe a day in your daily life.",
        "What does your day usually entail?",
        "How do you usually spend your day?",
        "Can you describe your daily life?",
        "Walk me through your average day.",
        "Walk me through your average day.",
        "Take me through your typical day.",
        "Talk me through your average day.",
        "What is your usual daily routine?",
        "What is your daily routine like?",
        "What is your daily routine like?",
        "What is a day in your life like?",
        "What's a typical day like?",
        "What's your daily routine like?",
        "What are your daily activities?",
        "Take me through your usual day.",
        "What takes up most of your day?",
        "What does your day consist of?",
        "Walk me through a regular day.",
        "What is a typical day for you?",
        "Detail your daily activities.",
        "How does your day usually go?",
        "Describe your daily schedule.",
        "What's your day-to-day like?",
        "What's a regular day entail?",
        "Describe a day in your life.",
        "How is your day structured?",
        "Outline your daily routine.",
        "Break down your day for me.",
        "What's an average day look?",
        "Describe your typical day.",
        "What does your day entail?",
        "How do you spend your day?",
        "What's a day like for you?",
        "Share your daily schedule.",
        "Narrate your daily events.",
        "What’s your daily routine?",
        "Walk me through your day.",
        "Take me through your day.",
        "How do you fill your day?",
        "Tell me about your day.",
        "Explain your usual day.",
        "Describe a day in life.",
        "How is your day spent?",
        "Walk me through a day.",
        "What fills your day?",
        "What is your day like?",
        "Describe your day?",
        "How's your day been?",
        "What's your typical day like?",
        "Your daily routine?",
        "What's a day like for you?",
        "What's your schedule like today?",
        "What's your day like?",
        "How has your day been so far?",
        "How do you spend your day?",
        "What's your usual day like?",
        "What does your day entail?",
        "What do you do during the day?",
        "Can you walk me through your day?",
        "How's your day shaping up?",
        "What have you been up to today?",
        "What does a typical day for you involve?",
        "What's on your agenda today?",
        "What's your day-to-day like?",
        "What's been the highlight of your day?",
        "How's your day progressing?",
        "What does your average day like?",
        "What's your daily schedule?",
        "What do you usually do during the day?",
        "How do you normally spend your day?",
        "What's your typical daily routine?",
    ])

    pprint(d_questions)
    print()
    pprint(d_questions_rev)


if __name__ == "__main__":
    main()



from typing import List
from typing import Optional

from .svc.generate_ttl_node import GenerateTTLNode
from .svc.generate_qa_pairs import GenerateQAPairs

qa_pair_generator = GenerateQAPairs().process
ttl_node_generator = GenerateTTLNode().process


def generate_qa_pairs(question: str,
                      answer: str) -> List[str]:
    """ Generate QA Pairs

    Args:
        question (str): a sample question
        answer (str): a sample answer

    Returns:
        List[str]: the Q&A Pairs
    """

    return qa_pair_generator(
        question=question,
        answer=answer)


def generate_node(entity_name: str,
                  entity_context: Optional[str] = None,
                  namespace: Optional[str] = "http://graffl.ai/test") -> str:
    """ Generate TTL Node

    Args:
        entity_name (str): the name of an entity
        entity_context (Optional[str], optional): the context this entity occurs. Defaults to None.
            providing a context will result in more precise synonyms
        namespace (str, optional): the OWL namespace. Defaults to "http://graffl.ai/test".

    Returns:
        str: a TTL node
    """

    return ttl_node_generator(
        entity_name=entity_name,
        entity_context=entity_context,
        namespace=namespace)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Generate Synonym Prompts """


from typing import Optional

from baseblock import BaseObject

from openai_helper import chat

from owl_builder.core import OpenAICache


class SynonymPromptGenerator(BaseObject):
    """ Generate Synonym Prompts """

    def __init__(self):
        """
        Created:
            29-Mar-2023
            craigtrim@gmail.com
            *   https://github.com/craigtrim/owl-builder/issues/10
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                entity_name: str,
                entity_context: Optional[str] = None,
                desired_synonym_size: Optional[str] = None) -> str:

        prompt_1 = f"Generate all the near English synonyms for '{entity_name}'"

        if not entity_context and not desired_synonym_size:
            return f"{prompt_1}."

        prompt_2 = f"where the synonym would make sense in the context '{entity_context}'."

        if entity_context and not desired_synonym_size:
            return f"{prompt_1} {prompt_2}"

        def get_size_prompt() -> str:

            if desired_synonym_size == 'unigram':
                return "Each synonym must be a unigram."

            if desired_synonym_size == 'bigram':
                return "Each synonym must be a bigram or smaller."

            if desired_synonym_size == 'trigram':
                return "Each synonym must be a trigram or smaller."

            raise NotImplementedError(desired_synonym_size)

        prompt_3 = get_size_prompt()

        if not entity_context and desired_synonym_size:
            return f"{prompt_1}. {prompt_3}"

        return f"{prompt_1} {prompt_2} {prompt_3}"



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" OpenAI: Generate Inflection Prompts """


from typing import Optional

from baseblock import BaseObject

from openai_helper import chat

from owl_builder.core import OpenAICache


class GenerateEnglishInflections(BaseObject):
    """ OpenAI: Generate Inflection Prompts """

    def __init__(self):
        """
        Created:
            20-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/5
        Updated:
            28-Mar-2023
            craigtrim@gmail.com
            *   complete re-write using openai-helper
                https://github.com/craigtrim/owl-builder/issues/8#issuecomment-1487312189
        """
        BaseObject.__init__(self, __name__)
        cache = OpenAICache()
        self._exists_in_cache = cache.exists
        self._write_to_cache = cache.write
        self._read_from_cache = cache.read

    def process(self,
                input_text: str,
                model: Optional[str] = "gpt-4") -> Optional[str]:

        if self._exists_in_cache(input_text):
            return self._read_from_cache(input_text)

        prompt = """
Generate all the English inflections for a word

Word: troubleshoot
Inflections: troubleshoots,troubleshooting,troubleshooted,troubleshooter,troubleshooters
Word: computer
Inflections: computers, computerized, computerizing, computerize, computerizes, computerizing, computerization, computerizations

Word: #INPUT_TEXT
Inflections:
        """.replace("#INPUT_TEXT", input_text)

        result = chat(input_prompt=prompt, messages=None,
                      remove_emojis=True, model=model)
        if not result or not len(result):
            return None

        result = [
            x.strip() for x in result.split(',')
            if x.lower() != input_text.lower()
        ]

        result = [
            x for x in result if x and len(x)
        ]

        self._write_to_cache(data=result, file_name=input_text)

        return result
